# this program takes avi files, and uses textgrids
# to chop the file up and format it for AAA software
#
# dependencies: PRAAT, ffmpeg, PERL, R, "reader", and "gdata"
#
# Copyright (c) Donald Derrick, Nov 2015
#

library(reader)
library(gdata)

options(useFancyQuotes = FALSE)

# Put your path in if you can't access ffmpeg from the command prompt
# if you don't know what a command prompt is, this code is not for you
ffmpegPath = "ffmpeg"

# Put your path in if you can't access PERL from the command prompt
perlPath = "perl"

# put in the name of the text grid labels to be used to obtain the positions of the videos to be chunked
# I highly recommend changing this to something like "word" - the example capture is a swallow for a palate trace
textGridLabel = "swallow"

# put in the type of video source material, usually avi or mov
videoInputType = "mov"

# put in the path of the video input
videoInputPath = "mov"

# put in the type of video you want to output - use AVI if you are doing this
# for AAA software
videoOutputType = "avi"

# Put in the path of the video output
videoOutputPath = "cropped"

# Put in the path of the directory holding the textgrids
textgridPath = "textgrid"

# put in the extension used for your PRAAT textgrids.  
textgridExtension = "textgrid"

# this is the part of the ultrasound imaging of the tongue video to crop out - to minimize the size for the tongue trace.

# This is the correct coordinates for the Logiq E ultrasound machine in B-mode.
cropRegion = "crop=695:470:204:55"


holdMakeDir = paste("mkdir -p ", videoOutputPath, sep="")
system(holdMakeDir,intern=TRUE, wait=TRUE)

#Loop through all of the files in the directory meant to contain textgrids.
for (i in list.files(path = textgridPath))
{
	#extension = tolower(get.ext(i))==textgridExtension
	fileName = rmv.ext(i,only.known=FALSE)
		
	# test if the file is a textgrid
	if (tolower(get.ext(i))==textgridExtension)
	{
		# if yes, create a csv file from the textgrid
		hold1 = dQuote(paste(getwd(), .Platform$file.sep, "textgrid2csv.pl", sep="")) 
		hold2 = dQuote(paste(getwd(), .Platform$file.sep, textgridPath, .Platform$file.sep , i, sep=""))
		holdPerl = paste(perlPath," ",hold1," ",hold2, sep="")
		system(holdPerl, intern=TRUE, wait=TRUE)
		
		# then copy the csv to the csv folder
		csvFile = paste(fileName, ".csv", sep="")

		movieFile = paste(fileName, ".", videoInputType,sep="")
		csvPath = (paste(getwd(), .Platform$file.sep, textgridPath, .Platform$file.sep, csvFile, sep=""))
		aviPath = (paste(getwd(),.Platform$file.sep, videoInputPath,.Platform$file.sep, movieFile, sep=""))
		
		# load the csv data
		csvData = read.csv(csvPath, sep=";", stringsAsFactors=FALSE)
		
		# set up counters
		k = 0
		l = 0
		# stop()
		for (j in csvData[,3])
		{
			k = k + 1
			# check to see if there is a label
			if (j == textGridLabel && csvData[k,6] != "")
			{
				# extract the start and end time
				timeStart = csvData[k,4]
				timeEnd = csvData[k,5]
		
				# give the label a numeric ordinal entry
				l = l + 1
				
				# 1) grap the avi file, crop it to Logiq E B-mode size
				# 2) clip start at the start time stamp
				# 3) clip end at the end of the segmentation duration
				# 4) name the file for the original file, ordinal entry,
				# and label.
				# 5) Save the file as a .avi, but make SURE it saves
				# the data as H.264 because other containers suck.
				cropBaseName = paste(fileName, "_", l, "_", csvData[k,6], sep="")
				hold3 = dQuote(paste(getwd(), .Platform$file.sep, videoInputType, .Platform$file.sep, fileName, ".",videoInputType, sep=""))
				hold4 = dQuote(paste(getwd(), .Platform$file.sep, videoOutputPath, .Platform$file.sep, cropBaseName, ".", videoOutputType, sep=""))
				
				holdCrop = paste(ffmpegPath, " -i ", hold3, " -filter:v '",cropRegion,"' -ss ", timeStart, " -t ", (timeEnd-timeStart), " -ac 1 -ar 44100 -acodec pcm_s16le -c:v libx264 -pix_fmt yuv420p ", hold4, sep="")				
				system(holdCrop, intern=TRUE, wait=TRUE)
			}
		}
		# delete the no-longer-needed file
		file.remove(csvPath)
	}
}



